#pragma once

enum check_t { MAX, MIN, AVG, NUM };

bool do_check(int rank, const std::vector<oglobal> &output, const std::string &name, check_t type, double value, size_t num = 0) {
    bool result = false;
    for (const auto &gl : output) {
        if (gl.header == name) {
            if (gl.values.size() == 0) {
                printx(rank, std::string("CHECK: No values for: ") + name);
                return false;;
            }
            double control_value = 0;
            std::string stype;
            switch (type) {
                case check_t::MAX: stype = "MAX"; control_value = *std::max_element(gl.values.begin(), gl.values.end()); result = true; break;
                case check_t::MIN: stype = "MIN"; control_value = *std::max_element(gl.values.begin(), gl.values.end()); result = true; break;
                case check_t::AVG: stype = "AVG"; control_value = std::reduce(gl.values.begin(), gl.values.end()) / (double)gl.values.size(); result = true; break;
                case check_t::NUM: stype = "NUM";
                                      if (gl.values.size() < num) {
                                          printx(rank, std::string("CHECK: Index out of bounds for: ") + name);
                                          break;
                                      }
                                      control_value = gl.values[num];
                                      result = true;
                                      break;
            }
            if (result && control_value < value) {
                printx(rank, std::string("CHECK: Failed for: ") + name + " type: " + stype + " expected: " + std::to_string(value) + " obtained: " + std::to_string(control_value));
                result = false;
            }
            return result;
        }    
    }
    return result;
}

bool do_all_checks(int rank, const std::vector<oglobal> &output) {
    /*

    rank=0: GPU_STREAM: {1279.1,1280.7,1245.8,1245.8}
    rank=0: H2D_D2H_pinned: {22.9,23.6,23.5,23.7,23.5,23.7,23.5,23.7}
    rank=0: H2D_D2H_pageable: {6.1,5.8,6.0,5.9,6.1,6.1,6.1,6.2}
    rank=0: MPI_P2P_intranode_GPU: {135.1,35.0,34.9}
    rank=0: MPI_P2P_crossnode_GPU: {21.3,13.6,21.3,14.3,21.3,13.6,21.2,22.3}
    rank=0: MPI_P2P_intranode_CPU: {18.9,18.7,18.0}
    rank=0: MPI_P2P_crossnode_CPU: {18.0,14.9,18.2,13.9,17.9,14.2,18.2,22.3}

    rank=0: GPU_STREAM: {1278.8,1280.7,1241.7,1234.3}
    rank=0: H2D_D2H_pinned: {22.8,23.5,23.5,23.7,23.4,23.7,23.4,23.7}
    rank=0: H2D_D2H_pageable: {12.7,11.7,13.0,12.5,13.0,12.9,13.1,13.4}

    rank=0: MPI_P2P_intranode_GPU_2DIR: {28.8,23.2,28.2,28.4}
    rank=0: MPI_P2P_crossnode_GPU_2DIR: {10.6,10.8,10.6,10.8}
    rank=0: MPI_P2P_intranode_GPU_1DIR: {34.7,38.4,32.0,47.8}
    rank=0: MPI_P2P_crossnode_GPU_1DIR: {11.2,11.2,11.2,11.2}
    rank=0: MPI_P2P_intranode_CPU_2DIR: {8.4,9.2,8.9,9.0}
    rank=0: MPI_P2P_crossnode_CPU_2DIR: {8.6,9.2,8.6,9.2}
    rank=0: MPI_P2P_intranode_CPU_1DIR: {16.0,17.3,18.2,16.7}
    rank=0: MPI_P2P_crossnode_CPU_1DIR: {7.8,11.2,10.2,11.2}

    */

    bool result = true;
    result = result & do_check(rank, output, "GPU_STREAM", AVG, 1200.0);
    result = result & do_check(rank, output, "GPU_STREAM", MIN, 1200.0);
    result = result & do_check(rank, output, "H2D_D2H_pinned", AVG, 18.0);
    result = result & do_check(rank, output, "H2D_D2H_pinned", MIN, 17.0);
    result = result & do_check(rank, output, "H2D_D2H_pageable", AVG, 11.0);
    result = result & do_check(rank, output, "H2D_D2H_pageable", MIN, 9.0);
#ifdef WITH_MPI
    result = result & do_check(rank, output, "MPI_P2P_intranode_GPU_1DIR", MAX, 34.0);
    result = result & do_check(rank, output, "MPI_P2P_intranode_GPU_1DIR", MIN, 30.0);
    result = result & do_check(rank, output, "MPI_P2P_intranode_CPU_1DIR", AVG, 16.0);
    result = result & do_check(rank, output, "MPI_P2P_intranode_CPU_1DIR", MIN, 15.0);
    result = result & do_check(rank, output, "MPI_P2P_crossnode_GPU_1DIR", MAX, 11.0);
    result = result & do_check(rank, output, "MPI_P2P_crossnode_GPU_1DIR", MIN, 10.0);
    result = result & do_check(rank, output, "MPI_P2P_crossnode_CPU_1DIR", MAX, 11.0);
    result = result & do_check(rank, output, "MPI_P2P_crossnode_CPU_1DIR", AVG, 9.0);

    result = result & do_check(rank, output, "MPI_P2P_intranode_GPU_2DIR", MAX, 27.0);
    result = result & do_check(rank, output, "MPI_P2P_intranode_GPU_2DIR", MIN, 21.0);

    result = result & do_check(rank, output, "MPI_P2P_intranode_CPU_2DIR", AVG, 8.5);
    result = result & do_check(rank, output, "MPI_P2P_intranode_CPU_2DIR", MIN, 7.0);

    result = result & do_check(rank, output, "MPI_P2P_crossnode_GPU_2DIR", MAX, 10.0);
    result = result & do_check(rank, output, "MPI_P2P_crossnode_GPU_2DIR", MIN, 9.0);

    result = result & do_check(rank, output, "MPI_P2P_crossnode_CPU_2DIR", MAX, 8.0);
    result = result & do_check(rank, output, "MPI_P2P_crossnode_CPU_2DIR", MIN, 7.0);
#endif
    if (!result) {
        printx(rank, "CHECK FAILED");
    }
    return result;
}

