#pragma once
#include <iostream>
#include <sstream>
#include <sys/time.h>

struct timer {
    std::stringstream comment;
    std::string name;
    size_t size;
    size_t niters;
    timeval tv[2];
    std::vector<double> &result;
    bool erased = false;
    timer(const std::string& _name, size_t _size, size_t _niters, std::vector<double> &_result) : name(_name), size(_size), niters(_niters), result(_result) { 
        gettimeofday(&tv[0], NULL); 
    }
    void start() { 
        gettimeofday(&tv[0], NULL); 
    }
    long time_diff() {
        return ((long)tv[1].tv_sec - (long)tv[0].tv_sec) * 1000000L + (long)tv[1].tv_usec - (long)tv[0].tv_usec;
    }
    void erase() {
        erased = true;
    }
    ~timer() {
        gettimeofday(&tv[1], NULL);
        long diff = 0;
        double bw = 0;
        if (!erased) {
            diff = time_diff();
            constexpr double GiB_const = (1<<30)/1000000;
            bw = (double)niters * (double)size / (double)(diff) / GiB_const;
        }
#ifdef WITH_OUTPUT
        std::cout << name << ": "
            << "[ "
            << "bandwidth(GiB/s): " << bw << ", "
            << "time(usec): " << diff/niters << ", "
            << "size: " << size;
        if (comment.str().size() != 0) {
            std::cout << ", comment: \"" << comment.str() << "\"";
        }
        std::cout << " ]" << std::endl;
#endif
        result.push_back(bw);
    }
};

